//// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
//// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
//// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//// PARTICULAR PURPOSE.
////
//// Copyright (c) Microsoft Corporation. All rights reserved

#include "pch.h"
#include "Material.h"
#include "Rendering/DeferredConstantBuffers.h"

using namespace DirectX;

//--------------------------------------------------------------------------------

Material::Material(
	XMFLOAT4 meshColor,
	XMFLOAT4 diffuseColor,
	XMFLOAT4 specularColor,
	float specularExponent,
	float meshColorBlendPercent,
	float specularColorBlendPercent,
	_In_ ID3D11ShaderResourceView* textureResourceView,
	_In_ ID3D11VertexShader* vertexShader,
	_In_ ID3D11PixelShader* pixelShader
	) :
	m_originalColor(meshColor),
	m_meshColor(meshColor),
	m_diffuseColor(diffuseColor),
	m_hitColor(),
	m_specularColor(specularColor),
	m_meshPercentDiffusePercent(meshColorBlendPercent, specularColorBlendPercent, 0.0f, 0.0f),
	m_specularExponent(specularExponent),
	m_vertexShader(vertexShader),
	m_pixelShader(pixelShader),
	m_textureRV(textureResourceView)
{
}

//--------------------------------------------------------------------------------

void Material::RenderSetup(
	_In_ ID3D11DeviceContext* context,
	_In_ DirectX::XMFLOAT4X4& world,
	_In_ ID3D11Buffer* primitiveConstantBuffer,
	_In_ bool isFirstPass
	)
{
	if (isFirstPass)
	{
		LightPrePassPerMesh constantBuffer;
		constantBuffer.World = world;
		constantBuffer.SpecularPower = m_specularExponent;

		context->UpdateSubresource(primitiveConstantBuffer, 0, nullptr, &constantBuffer, 0, 0);
		context->PSSetShaderResources(0, 1, m_textureRV.GetAddressOf());
		context->VSSetShader(m_vertexShader.Get(), nullptr, 0);
		context->PSSetShader(m_pixelShader.Get(), nullptr, 0);
	}
	else
	{
		LightFinalPassPerMesh constantBuffer = {};
		constantBuffer.DiffuseTint = m_diffuseColor;
		constantBuffer.SpecularColor = m_specularColor;
		constantBuffer.MeshColor = m_meshColor;
		constantBuffer.World = world;
		constantBuffer.OverridePercentDiffusePercent = m_meshPercentDiffusePercent;
		context->UpdateSubresource(primitiveConstantBuffer, 0, nullptr, &constantBuffer, 0, 0);
		context->PSSetShaderResources(1, 1, m_textureRV.GetAddressOf());
	}
}